package com.perforce.spark;

import java.util.Map;

import spark.ModelAndView;
import spark.Spark;
import spark.template.freemarker.FreeMarkerEngine;

import com.perforce.spark.browse.BrowseControl;
import com.perforce.spark.configuration.ConfigControl;
import com.perforce.spark.connection.ConnectionSession;
import com.perforce.spark.deploy.DeployControl;
import com.perforce.spark.depot.DepotControl;
import com.perforce.spark.navigation.NavBar;
import com.perforce.spark.search.SearchControl;
import com.perforce.spark.view.ViewControl;

public class Main {

	public static void main(String[] args) throws Exception {

		Spark.staticFileLocation("/public");

		Spark.before((request, response) -> {
			ConnectionSession.basicAuth(request, response);
			if (!ConnectionSession.isValid(request)) {
				ConnectionSession.cookie(request, response);
			}
		});

		Spark.get("/", (request, response) -> {
			Map<String, Object> attributes;
			attributes = NavBar.attributes("cover", request);
			return new ModelAndView(attributes, "page-cover.html");
		}, new FreeMarkerEngine());

		Spark.post("/login", (request, response) -> {
			return ConnectionSession.login(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/logout", (request, response) -> {
			ConnectionSession.logout(request, response);
			response.redirect("/");
			return null;
		});

		Spark.get("/list", (request, response) -> {
			return BrowseControl.list(request, response);
		}, new FreeMarkerEngine());
		
		Spark.get("/list/", (request, response) -> {
			return BrowseControl.list(request, response);
		}, new FreeMarkerEngine());
		
		Spark.get("/browse", (request, response) -> {
			return BrowseControl.depots(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/browse/", (request, response) -> {
			return BrowseControl.depots(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/browse/*", (request, response) -> {
			return BrowseControl.browse(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/view/*", (request, response) -> {
			return ViewControl.view(request, response);
		});

		Spark.get("/search", (request, response) -> {
			return SearchControl.query(request, response);
		}, new FreeMarkerEngine());
		
		Spark.get("/config", (request, response) -> {
			return ConfigControl.listDepots(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/create/depot/:type/:site", (request, response) -> {
			return DepotControl.getCreateDepot(request, response);
		}, new FreeMarkerEngine());

		Spark.post("/create/depot", (request, response) -> {
			DepotControl.postCreateDepot(request, response);
			response.redirect("/config");
			return "";
		});
		
		Spark.get("/edit/depot/:repo", (request, response) -> {
			return DepotControl.getEditDepot(request, response);
		}, new FreeMarkerEngine());

		Spark.get("/deploy/:type", (request, response) -> {
			return DeployControl.getDeploy(request, response);
		}, new FreeMarkerEngine());

		Spark.post("/deploy/:type", (request, response) -> {
			return DeployControl.postDeploy(request, response);
		});

		Spark.get("/submit/:type", (request, response) -> {
			return DeployControl.getSubmit(request, response);
		}, new FreeMarkerEngine());

		Spark.post("/submit/:type", (request, response) -> {
			return DeployControl.postSubmit(request, response);
		});
		
		Spark.get("/report/:type/*", (request, response) -> {
			return DeployControl.getReport(request, response);
		}, new FreeMarkerEngine());
	}
}
