package com.perforce.spark.artifact.proxy;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.fileupload.FileUploadException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import spark.Request;

import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.spark.artifact.DefaultArtifact;
import com.perforce.spark.artifact.ArtifactFactory;
import com.perforce.spark.artifact.ArtifactInterface;
import com.perforce.spark.depot.DepotFactory;
import com.perforce.spark.depot.DepotModel;

public class ProxyArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(ProxyArtifact.class);

	private URL proxy;
	private Path path;

	public ProxyArtifact(IOptionsServer p4, URL proxy, String depot, String path) {
		super(p4);

		this.proxy = proxy;
		this.path = Paths.get(path);
		this.repoID = depot;
	}

	@Override
	public Path getPath() {
		String groupID = path.getParent().toString();
		String id[] = { groupID };
		Path depotPath = Paths.get(repoID, id);
		return depotPath;
	}

	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public List<String> populateWorkspace(File root) throws Exception {
		List<String> list = new ArrayList<>();
		URL url = new URL(proxy, path.toString());
		Path filename = path.getFileName();
		Path filepath = Paths.get(root.toString(), filename.toString());

		logger.info("proxy loading: " + url + " (" + filepath + ")");
		InputStream in = url.openStream();
		Files.copy(in, filepath);
		in.close();

		list.add(filepath.toString());
		return list;
	}

	@Override
	public void processForm(Request request) throws FileUploadException {
		// No form to process
	}

	@Override
	public void index() throws Exception {
		ArtifactInterface artifact = ArtifactFactory.getProxy(p4, repoID);
		artifact.index();
	}

	@Override
	public void buildIndex() throws Exception {
		// no indexing
	}
}
